from typing import List

from pydantic import BaseModel, constr

# Order of models is defined by the order of occurrence in the self_disclosure plugin

NonEmptyStr = constr(min_length=1)
NoStarStr = constr(regex=r"^[^*]+$")


class User(BaseModel):  # response model
    user_id: str
    username: str
    firstname: str
    lastname: str
    type: str
    school_id: str

    class Config:
        schema_extra = {
            "example": {
                "user_id": "d8aa843b-9628-42b8-9637-d198c745efa5",
                "username": "m.mustermann",
                "firstname": "Max",
                "lastname": "Mustermann",
                "type": "teacher",
                "school_id": "SchoolAuthority-School1",
            }
        }


class Schoolclass(BaseModel):
    group_id: str
    name: str


class CountedSchoolclass(Schoolclass):
    student_count: int


class UsersClasses(BaseModel):  # response model
    groups: List[CountedSchoolclass]

    class Config:
        schema_extra = {
            "example": {
                "groups": [
                    {
                        "group_id": "503f1278-9cda-49f6-a749-3af022a7d32b",
                        "name": "1a",
                        "student_count": 2,
                    },
                    {
                        "group_id": "3be3c78b-6263-4c42-9d5e-14ec95f0c849",
                        "name": "2b",
                        "student_count": 4,
                    },
                ]
            }
        }


class Teacher(BaseModel):
    user_id: str
    username: str


class Student(Teacher):
    firstname: str
    lastname: str


class ClassMembers(BaseModel):  # response model
    students: List[Student]
    teachers: List[Teacher]

    class Config:
        schema_extra = {
            "example": {
                "students": [
                    {
                        "user_id": "0e49f8b0-9c98-4716-9fe1-23d4466d9af8",
                        "username": "jane.doe",
                        "firstname": "Jane",
                        "lastname": "Doe",
                    },
                    {
                        "user_id": "86f753e0-50b3-44cb-807e-2c2556b0e6ca",
                        "username": "john.doe",
                        "firstname": "John",
                        "lastname": "Doe",
                    },
                ],
                "teachers": [
                    {"user_id": "d8aa843b-9628-42b8-9637-d198c745efa5", "username": "m.mustermann"}
                ],
            }
        }


class Schoolclasses(BaseModel):  # response model
    groups: List[Schoolclass]


class UdmProperties(BaseModel):
    brokerID: str


class BrokeridUser(BaseModel):
    """Describes a user resource as returned by the kelvin api.
    Only contains fields that we actually use, so that we don't
    need to completely sync model descriptions from both sides.
    """

    udm_properties: UdmProperties
    name: str
    firstname: str
    lastname: str

    class Config:
        orm_mode = True
        extra = "allow"
